/*!
    \defgroup sensor Sensor
    \brief This module receives GPU data.

     This module receives GPU data.
*/
///@{

#pragma once

#include <ObSensor.h>
#include <nvml.h>
#include <pstreams/pstream.h>

///
/// \class ObSensorGpu
///
/// \brief GPU Obmon sensor class
/// \author Martin Vala <mvala@cern.ch>
/// \author Branislav Beke <beke.private@yandex.com>
///

class ObSensorGpu : public ObSensor {
public:
  ObSensorGpu(std::string _name = {"gpu"});
  virtual ~ObSensorGpu() override;

  bool init() override;
  void process() override;
  void speed(ObSensor *s1, ObSensor *s2, unsigned int timeout = 1000) override;
  std::string json(const std::string name = {"static"}) const override;

private:
  std::string getProcUser(unsigned int) const;

  /// Structure for device stat simplification
  typedef struct devStat_t {
    std::string name;       ///< Device name reported by NVML
    nvmlUtilization_t util; ///< GPU utilization stats
    nvmlMemory_t mem;       ///< GPU memory stats
  } devStat_t;

  /// Count of NVIDIA devices
  unsigned int _count = 0;

  /// NVML device handles
  std::vector<nvmlDevice_t> fDevices;

  /// Device statistics
  std::vector<devStat_t> fStatus;

  /// Compute processes (ex. CUDA Applications)
  std::vector<std::vector<nvmlProcessInfo_t>> fComputeProc;

  /// Graphics processes (ex. desktop env)
  std::vector<std::vector<nvmlProcessInfo_t>> fGraphicsProc;
};

///@}
